-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

M.values = {}
local lastValues = {}
M.disabledState = {}

local smoothers = {
  wheelspeed = newExponentialSmoothing(10),
  gear_A = newExponentialSmoothing(10),
  --gear_M = newExponentialSmoothing(10),
  rpm = newExponentialSmoothing(10),
  lights = newExponentialSmoothing(10),
  fuel = newExponentialSmoothing(100),
  oiltemp = newExponentialSmoothing(100),
  watertemp = newExponentialSmoothing(100),
  turnsignal = newExponentialSmoothing(10),
  airspeed = newExponentialSmoothing(10),
  altitude = newExponentialSmoothing(10)
}

local lightsState = 0

local signalRightState = false
local signalLeftState = false
local signalWarnState = false
local lightbarState = 0

local fogLightsState = false

local blinkPulse = false
local blinkTimerThreshold = 0.4
local blinkTimer = 0
local rpmspin = 0

local indicatorLoopSound = nil

local hasSteered = false -- used to see whether right/left-turn has been finished

-- set to nop in the beginning - this avoids conflict with the warn signal
local automatic_indicator_stop = nop

local function generateBlinkPulse(dt)
  blinkTimer = blinkTimer + dt
  if blinkTimer > blinkTimerThreshold then
    blinkPulse = not blinkPulse
    blinkTimer = 0
  end
end

local function playIndicatorStartSound()
  if v.data.refNodes then
    --sounds.playSoundOnceAtNode("indicator_default_start", v.data.refNodes[0].ref, 1)
  end
end

local function playIndicatorStopSound()
  if v.data.refNodes then
    --sounds.playSoundOnceAtNode("indicator_default_stop", v.data.refNodes[0].ref, 1)
  end
end

-- stops automatically indicator if turn has been finished or if wheel is steered in opposite direction
local isBlinkerOn = 0 --//////////////////
local IsWarnOn = 0    --//////////////////
local OnOff = 0       --//////////////////

local function manage_automatic_indicator_stop()
  local controlPoint = 100
  local steering = M.values.steering
  if steering == nil then return end

  --check whether user has steered in the desired direction
  if signalLeftState and steering > controlPoint then
    hasSteered = true
  elseif signalRightState and steering < -controlPoint then
    hasSteered = true
  end

  --hasSteered = ((signalRightState and steering < -controlPoint) or (signalLeftState and steering > controlPoint))

  --if the wheel has returned to the neutral position, turn indicator off

  
  if signalLeftState and hasSteered and steering <=0 then
    signalLeftState = false
    hasSteered = false
    playIndicatorStopSound()
    automatic_indicator_stop = nop
	sounds.blinkerOff() --//////////////////
	isBlinkerOn = 0     --//////////////////
  elseif signalRightState and hasSteered and steering >= 0 then
    signalRightState = false
    hasSteered = false
    playIndicatorStopSound()
    automatic_indicator_stop = nop
	sounds.blinkerOff() --//////////////////
	isBlinkerOn = 0     --//////////////////
  end
end


-- user input functions
local function toggle_left_signal()
  if not signalWarnState then
    signalLeftState = not signalLeftState
  else
    signalLeftState = true
  end
  if signalLeftState then
    signalRightState = false
    signalWarnState = false
    playIndicatorStartSound()
    automatic_indicator_stop = manage_automatic_indicator_stop
	if sounds.cameraMode == "dash" then sounds.blinkerOn() IsWarnOn = 0 isBlinkerOn = 1 end --//////////////////
	
  end
  if not signalLeftState then
    playIndicatorStopSound()
    automatic_indicator_stop = nop
	sounds.blinkerOff() --//////////////////
	isBlinkerOn = 0     --//////////////////
  end
end

local function toggle_right_signal()
  if not signalWarnState then
    signalRightState = not signalRightState
  else
    signalRightState = true
  end
  if signalRightState then
    signalLeftState = false
    signalWarnState = false
    playIndicatorStartSound()
    automatic_indicator_stop = manage_automatic_indicator_stop
	if sounds.cameraMode == "dash" then sounds.blinkerOn() IsWarnOn = 0 isBlinkerOn = 1 end --//////////////////
  end
  if not signalRightState then
    automatic_indicator_stop = nop
    playIndicatorStopSound()
	sounds.blinkerOff() --//////////////////
	isBlinkerOn = 0     --//////////////////
  end
end


local function update(dt)
  generateBlinkPulse(dt)

  local vals = M.values
  -- the primary source values

  automatic_indicator_stop()

  rpmspin = rpmspin + (dt * (vals.rpm or 0))
  if rpmspin > 360 then rpmspin = rpmspin - 360 end
  vals.rpmspin = rpmspin

  vals.parkingbrake = vals.parkingbrake_input
  vals.lights = lightsState
  vals.lights_state = lightsState
  if signalWarnState then vals.turnsignal = 0
  elseif signalRightState then vals.turnsignal = 1
  elseif signalLeftState then vals.turnsignal = -1
  else vals.turnsignal = 0
  end

  local vel = obj:getVelocity()
  vals.airspeed = vel:length()
  vals.virtualAirspeed = sensors.virtualAirspeed
  vals.verticalVelocity = vel.z
  vals.altitude = obj:getPosition().z
  vals.parking = 0 -- TODO: input.parkinglights
  vals.reverse = (vals.gearIndex or 0) < 0

  -- and then the derived values
  vals.signal_L = (vals.signal_left_input == 1 and blinkPulse)
  vals.signal_R = (vals.signal_right_input == 1 and blinkPulse)
  vals.signal_right_input = (signalRightState)
  vals.signal_left_input = (signalLeftState)
  vals.hazard = (signalWarnState and blinkPulse)
  vals.hazard_enabled = signalWarnState
  vals.lightbar = lightbarState
  vals.lowpressure = (beamstate.lowpressure)
  vals.oil = (vals.oiltemp or 0) >= 130
  vals.lowhighbeam = (lightsState == 1 or lightsState == 2)
  vals.lowbeam = (lightsState == 1)
  vals.highbeam = (lightsState == 2)
  vals.fog = fogLightsState

  -- inject imported electrics events
  beamstate.updateRemoteElectrics()

  for f, v in pairs(vals) do
    if M.disabledState[f] ~= nil then
      vals[f] = nil
    else
      if type(v) == 'boolean' then
        vals[f] = vals[f] and 1 or 0
      end
    end
  end

  for f, s in pairs(smoothers) do
    if vals[f] ~= nil then
      vals[f] = s:get(vals[f])
    end
  end

  for f, v in pairs(vals) do
    if v ~= lastValues[f] then
      material.funcChanged(f, v)
    end
    lastValues[f] = v
  end

  --obj:setVolume(indicatorLoopSound, math.max(vals.signal_L, vals.signal_R))
  gui.send("electrics", vals)
  
  local test = obj:queueGameEngineLua("be:getPlayerVehicle(0):queueLuaCommand('sounds.cameraMode = (\"'..camera.getActiveMode(0)..'\")')") --//////////////////
  if sounds.cameraMode ~= "dash" then sounds.blinkerOff() end                                                                              --//////////////////
  if sounds.cameraMode == "dash" then if isBlinkerOn == 1 or IsWarnOn == 1 then sounds.blinkerOn() end end                                 --//////////////////
  
end

local function reset()
  M.disabledState = {}

  for _,s in pairs(smoothers) do
    s:set(0)
  end

  if not indicatorLoopSound then
    --indicatorLoopSound = sounds.createSFXSource("art/sound/indicator_default_loop.ogg", 'AudioDefaultLoop3D', "", v.data.refNodes[0].ref)
	combustionEngineThermals = require("lua/vehicle/powertrain/combustionEngineThermals") --//////////////////
	sounds.smokeOff()																	  --//////////////////
	combustionEngineThermals.isSmoke = 0												  --//////////////////
	powertrain.isBroken = 0
  end

  M.values.throttle = 0
  M.values.brake = 0
  M.values.steering = 0
  M.values.clutch = 0
  M.values.wheelspeed = 0
end

local function beamBroke(id)
--[[
  if not v.data.props then return end
  local brokenBeamName = v.data.beams[id].name
  if not brokenBeamName then
    return
  end

  for propKey, prop in pairs (v.data.props) do
    if prop.disableBeams then
      for k,ab in pairs(prop.disableBeams) do
        if ab == brokenBeamName then
          log('D', "electrics.beamBroke", "prop beam broke:"..v)
          prop.disabled = 1
          break
        end
      end
    end
  end
]]--
end

local function set_warn_signal(value)
  signalWarnState = value
  signalRightState = signalWarnState
  signalLeftState  = signalWarnState
end

local function toggle_warn_signal()
  set_warn_signal(not signalWarnState) 
  if IsWarnOn == 0 then IsWarnOn = 1 isBlinkerOn = 0 sounds.blinkerOn()          --//////////////////
  elseif IsWarnOn == 1 then IsWarnOn = 0 sounds.blinkerOff() end                 --////////////////// 
end

local function toggle_lights()
  lightsState = lightsState + 1
  if lightsState == 3 then lightsState = 0 end
  sounds.loadSounds()
  sounds.playSoundOnceAtNode("lights", v.data.refNodes[0].ref, 0.4) --//////////////////
end

local function toggle_lightbar_signal()
  if sounds.isOn == 1 then          --//////////////////
  sounds.policeOn()                 --//////////////////
  end                               --//////////////////
  if lightbarState ~= 1 then        --//////////////////
  sounds.policeOn()	                --//////////////////
  end                               --//////////////////
  lightbarState = lightbarState + 1
  if lightbarState >= 2 then lightbarState = 0 end
end

local function toggle_lightbar_signalonly()
  lightbarState = lightbarState + 1
  if lightbarState >= 2 then lightbarState = 0 end
  if sounds.isOn == 1 then --//////////////////
  sounds.policeOn()        --//////////////////
  end
end

local function set_lightbar_signal(state)
  lightbarState = state
  if lightbarState >= 2 then lightbarState = 0 end
end

local function setLightsState(newval)
  lightsState = newval
end

local function toggle_fog_lights()
  fogLightsState = not fogLightsState
  sounds.loadSounds()
  sounds.playSoundOnceAtNode("lights", v.data.refNodes[0].ref, 0.4) --//////////////////
end

local function set_fog_lights(state)
  fogLightsState = state
end

-- public interface
M.update = update
M.toggle_left_signal        = toggle_left_signal
M.toggle_right_signal       = toggle_right_signal
M.toggle_warn_signal        = toggle_warn_signal
M.set_warn_signal           = set_warn_signal
M.toggle_lightbar_signal    = toggle_lightbar_signal
M.toggle_lightbar_signalonly= toggle_lightbar_signalonly
M.set_lightbar_signal       = set_lightbar_signal
M.toggle_fog_lights         = toggle_fog_lights
M.set_fog_lights            = set_fog_lights
M.toggle_lights             = toggle_lights
M.setLightsState            = setLightsState
M.beamBroke                 = beamBroke
M.reset                     = reset
M.init                      = reset

return M
